@@ -95,7 +95,17 @@ impl MariaDb {
 pub struct RunningMySqlService<'a>(GraphNodeReference, &'a ());
 
 impl MySqlService {
+    /// Makes sure mysqld is running (see [ServiceRunning::is_running])
     pub fn run<R: Requirement>(&mut self, context: &mut Context<R>) -> RunningMySqlService
+    where
+        R: Supports<ServiceRunning>,
+    {
+        let node = ServiceRunning::is_running(context, &self.service);
+        RunningMySqlService(node, &())
+    }
+
+    /// Restarts mysqld (see [ServiceRunning::restart])
+    pub fn restart<R: Requirement>(&mut self, context: &mut Context<R>) -> RunningMySqlService
     where
         R: Supports<ServiceRunning>,
     {
@@ -577,7 +587,12 @@ impl Display for CreateMySqlGrant {
 
 #[cfg(test)]
 mod tests {
-    use crate::builder::mysql::{CreateMySqlDatabase, CreateMySqlGrant, CreateMySqlUser};
+    use crate::{
+        builder::mysql::{CreateMySqlDatabase, CreateMySqlGrant, CreateMySqlUser},
+        requirements::Requirement,
+        system::System,
+        testing::LxcInstance,
+    };
 
     #[test]
     pub fn serialize_deserialize_create_mysql_database() {
@@ -590,6 +605,31 @@ mod tests {
         assert_eq!(r, serde_json::from_str(json).unwrap());
     }
 
+    #[test]
+    #[ignore]
+    pub fn lxc_create_mysql_database() {
+        let mut sys = LxcInstance::start();
+        let p = CreateMySqlDatabase {
+            name: String::from("foo"),
+        };
+
+        sys.execute_command("apt-get", &["install", "-y", "mariadb-server"])
+            .unwrap();
+
+        assert!(!p.has_been_created(&mut sys).unwrap());
+        assert!(!p.verify(&mut sys).unwrap());
+
+        p.create(&mut sys).unwrap();
+
+        assert!(p.has_been_created(&mut sys).unwrap());
+        assert!(p.verify(&mut sys).unwrap());
+
+        // p.delete(&mut sys).unwrap();
+
+        // assert!(!p.has_been_created(&mut sys).unwrap());
+        // assert!(!p.verify(&mut sys).unwrap());
+    }
+
     #[test]
     pub fn serialize_deserialize_create_mysql_user() {
         let r = CreateMySqlUser {
@@ -602,6 +642,39 @@ mod tests {
         assert_eq!(r, serde_json::from_str(json).unwrap());
     }
 
+    #[test]
+    #[ignore]
+    pub fn lxc_create_mysql_user() {
+        let mut sys = LxcInstance::start();
+        let p = CreateMySqlUser {
+            name: String::from("foo"),
+            pass: String::from("bar"),
+        };
+
+        sys.execute_command("apt-get", &["install", "-y", "mariadb-server"])
+            .unwrap();
+
+        assert!(!p.has_been_created(&mut sys).unwrap());
+        assert!(!p.verify(&mut sys).unwrap());
+
+        p.create(&mut sys).unwrap();
+
+        assert!(p.has_been_created(&mut sys).unwrap());
+        assert!(p.verify(&mut sys).unwrap());
+
+        assert!(
+            sys.execute_command_with_input("mysql", &["-ufoo", "-pbar"], "SELECT 1;".as_bytes())
+                .unwrap()
+                .is_success(),
+            "User was not created correctly"
+        );
+
+        p.delete(&mut sys).unwrap();
+
+        assert!(!p.has_been_created(&mut sys).unwrap());
+        assert!(!p.verify(&mut sys).unwrap());
+    }
+
     #[test]
     pub fn serialize_deserialize_create_mysql_grant() {
         let r = CreateMySqlGrant {
@@ -614,4 +687,58 @@ mod tests {
         assert_eq!(serde_json::to_string(&r).unwrap(), json);
         assert_eq!(r, serde_json::from_str(json).unwrap());
     }
+
+    #[test]
+    #[ignore]
+    pub fn lxc_create_mysql_grant() {
+        let mut sys = LxcInstance::start();
+        let pre1 = CreateMySqlUser {
+            name: String::from("foo"),
+            pass: String::from("bar"),
+        };
+        let pre2 = CreateMySqlDatabase {
+            name: String::from("baz"),
+        };
+        let p = CreateMySqlGrant {
+            user: String::from("foo"),
+            database: String::from("baz"),
+            privileges: String::from("SELECT"),
+        };
+
+        sys.execute_command("apt-get", &["install", "-y", "mariadb-server"])
+            .unwrap();
+
+        // Check when user and db don't exist
+        assert!(!p.has_been_created(&mut sys).unwrap());
+        assert!(!p.verify(&mut sys).unwrap());
+
+        pre1.create(&mut sys).unwrap();
+
+        // Check when only user exists
+        assert!(!p.has_been_created(&mut sys).unwrap());
+        assert!(!p.verify(&mut sys).unwrap());
+
+        pre1.delete(&mut sys).unwrap();
+        pre2.create(&mut sys).unwrap();
+
+        // Check when only db exists
+        assert!(!p.has_been_created(&mut sys).unwrap());
+        assert!(!p.verify(&mut sys).unwrap());
+
+        pre1.create(&mut sys).unwrap();
+
+        // Check when both db and user exist
+        assert!(!p.has_been_created(&mut sys).unwrap());
+        assert!(!p.verify(&mut sys).unwrap());
+
+        p.create(&mut sys).unwrap();
+
+        assert!(p.has_been_created(&mut sys).unwrap());
+        assert!(p.verify(&mut sys).unwrap());
+
+        p.delete(&mut sys).unwrap();
+
+        assert!(!p.has_been_created(&mut sys).unwrap());
+        assert!(!p.verify(&mut sys).unwrap());
+    }
 }
